/* GtkDatabox - An extension to the gtk+ library
 * Copyright (C) 1998 - 2005  Dr. Roland Bock
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

/**
 * @file gtkdatabox.h
 * GtkDatabox Interface.
 * 
 * Main interface to the GtkDatabox library
 */

#ifndef __GTK_DATABOX_H__
#define __GTK_DATABOX_H__

#include <gdk/gdk.h>
#include <gtk/gtkvbox.h>	/* We need this to pack a table containing the data, */

#ifdef __cplusplus
extern "C"
{
#endif				/* __cplusplus */

#define GTK_TYPE_DATABOX            (gtk_databox_get_type ())
#define GTK_DATABOX(obj)            GTK_CHECK_CAST (obj, gtk_databox_get_type (), GtkDatabox)
#define GTK_DATABOX_CLASS(klass)    GTK_CHECK_CLASS_CAST (klass, gtk_databox_get_type (), GtkDataboxClass)
#define GTK_IS_DATABOX(obj)         GTK_CHECK_TYPE (obj, gtk_databox_get_type ())
#define GTK_IS_DATABOX_CLASS(klass) GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_DATABOX)
#define GTK_DATABOX_GET_CLASS(obj)        (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_DATABOX, GtkDataboxClass))


   typedef struct _GtkDatabox GtkDatabox;
   typedef struct _GtkDataboxClass GtkDataboxClass;

   /**
    * GtkDatabox types of displays for data.
    *
    * GtkDatabox can display data in a number of ways (e.g. as points, 
    * lines, bars). The different types of data display are 
    * identified by an index. The indices for the display types which 
    * come with the source of the widget are defined in this 
    * enumeration. It is possible to add more display types.
    *
    * @see gtk_databox_display_type_add for adding more display types
    */
   typedef enum
   {
      GTK_DATABOX_NOT_DISPLAYED = 0,	
            /**< If data is configured with this diplay type, 
             * it is not displayed at all 
             * (you may want to use this for scaling) 
             */
      GTK_DATABOX_POINTS,
            /**< Display data as points.
             * 
             * @see gtk_databox_points_draw
             */
      GTK_DATABOX_LINES,
            /**< Display data as lines connecting the data points. 
             * 
             * @see gtk_databox_lines_draw
             */
      GTK_DATABOX_BARS,
            /**< Display data as vertical lines/bars between the X axis and 
             * the data points. 
             * 
             * @see gtk_databox_bars_draw
             */
      GTK_DATABOX_GRID,
            /**< With this type, the data itself is not displayed, but 
             * a grid like in an oscilloscope is shown. 
             * 
             * @see gtk_databox_grid_draw
             * @see gtk_databox_grid_set_config
             */
      GTK_DATABOX_CROSS_SIMPLE,
            /**< With this type, the data itself is not displayed, but 
             * a very simple coordinate cross is shown.
             * 
             * @see gtk_databox_cross_simple_draw
             */
      GTK_DATABOX_MARKER,
            /**< With this type, various kinds of markers (optionally with 
             * textual description) are drawn, which mark the data points.
             * 
             * @see gtk_databox_marker_draw
             * @see gtk_databox_marker_set_markers
             */
      GTK_DATABOX_TEXT,
            /**< Displays configurable texts next to the data points.
             * 
             * @see gtk_databox_text_draw
             */
   } GtkDataboxDisplayType;
   
   /**< GtkDataboxData is a structure which contains the data itself and 
    * the necessary information for displaying it.
    *
    * @see gtk_databox_points_draw
    * @see gtk_databox_lines_draw
    * @see gtk_databox_grid_draw
    */
   typedef struct
   {
      gfloat *X;		/**< X (horizontal) values.
                                 *
                                 * This array has to have at least as 
                                 * many gfloat values as indicated by 
                                 * "length" (more values don't hurt, 
                                 * but they aren't diplayed either).
                                 *
                                 * Pairs of (X[i],Y[i]) represent data 
                                 * points. Most drawing functions interpret 
                                 * this a horizontal and vertical coordinates.
                                 * Exceptions are special drawing functions:
                                 *
                                 * @see gtk_databox_grid_draw
                                 * @see gtk_databox_cross_simple_draw
                                 *
                                 */
      gfloat *Y;		/**< Y (vertical) values.
                                 *
                                 * @see X
                                 */
      guint length;		/**< Number of data points. 
                                 *
                                 * This number indicates how many values
                                 * are contained in both X and Y. The 
                                 * drawing functions rely on this and 
                                 * don't take any measures to verify the 
                                 * length.
                                 */

      guint size;		/**< Size of data representation. 
                                 *
                                 * If drawing functions have scalable sizes
                                 * of data representations (e.g. line width,
                                 * size of dots, etc), this value tells the 
                                 * function how small or big the representation
                                 * should be.
                                 */
      GdkGC *gc;		/**< Graphics Context for data. 
                                 * 
                                 * The gc contains information about color, 
                                 * line styles, etc.. It is 
                                 * used by the drawing function to display 
                                 * the data (X and Y).
                                 */
      
      gpointer draw_info;       /**< Information for the drawing function.
                                 * 
                                 * Data stored at this address is handled 
                                 * and destroyed by functions related to the 
                                 * drawing function, for instance:
                                 * 
                                 * @see gtk_databox_marker_set_markers
                                 * @see gtk_databox_marker_cleanup
                                 */
   }
   GtkDataboxData;
   
   typedef void (*GtkDataboxDrawFunc) (GtkDatabox * box, 
                                       GtkDataboxData * data);
   typedef void (*GtkDataboxRemoveFunc) (GtkDatabox * box, 
                                         GtkDataboxData * data);
   #define GTK_DATABOX_DRAW_FUNC(f)           ((GtkDataboxDrawFunc) (f))
   #define GTK_DATABOX_REMOVE_FUNC(f)         ((GtkDataboxRemoveFunc) (f))
   
   typedef struct
   {
      gint x;
      gint y;
   }
   GtkDataboxCoord;		/* Pixel coordinates */

   typedef struct
   {
      gfloat x;
      gfloat y;
   }
   GtkDataboxValue;		/* Data coordinates */

   struct _GtkDatabox
   {
      GtkVBox box;

      /* Table containing the display, scrollbars and rulers */
      GtkWidget *table;

      /* This list contains the data sets that are to be drawn. */
      GList *data_data;

      /* This is where we display the data */
      GtkWidget *draw;

      /* The rulers */
      GtkWidget *rulerX;
      GtkWidget *rulerY;

      /* The scrollbars and their adjustments */
      GtkWidget *scrollbarX;
      GtkWidget *scrollbarY;
      GtkAdjustment *adjX;
      GtkAdjustment *adjY;

      /* The pixmap for double buffering */
      GdkPixmap *pixmap;

      /* A number of flags (zoom enabled/disabled, etc.) */
      gulong flags;

      /* An array of max_points points */
      GdkPoint *points;

      /* Number of values of the largest data set currently in the databox */
      guint max_points;

      /* The selection box is drawn using this gc. You can manipulate it
       * by using 
       * gtk_databox_show_selection_filled or 
       * gtk_databox_hide_selection_filled
       */
      GdkGC *select_gc;

      /* Size (in pixel) of the display area */
      GtkDataboxCoord size;

      /* Position (pixel) of the last mouse click */
      GtkDataboxCoord marked;

      /* During creation of a selection box this is the second corner of
       * the box, opposed to marked
       */
      GtkDataboxCoord select;

      /* The extrema of the data values. These are calculated when 
       * gtkdatabox_rescale is called. You can set them directly via
       * gtkdatabox_rescale_with_values.
       */
      GtkDataboxValue min;
      GtkDataboxValue max;

      /* The visible extrema in the top-left and bottom-right corner.
       * These are influenced by zooming and scrolling.
       */
      GtkDataboxValue top_left;
      GtkDataboxValue bottom_right;

      /* Factor for translation of data values to screen coordinates
       * and vice versa. This is influenced by zooming and the extrema of
       * the data. You can use gtk_databox_rescale_with_values for
       * manipulating the extrema.
       */
      GtkDataboxValue factor;

      /* If we zoom to far into the data, we will get funny results, because
       * of overflow effects. Therefore zooming has to be limited.
       * The default zoom limit is 0.01, meaning that you can magnify your
       * data by a factor of 100.
       * Use gtk_set_zoom_limit to change this value.
       */
      gfloat zoom_limit;

      /* If we want to use a different cursor, we will store it here */
      GdkCursor *cursor;
   };

   struct _GtkDataboxClass
   {
      GtkVBoxClass parent_class;

      void (*gtk_databox) (GtkDatabox * box);

      /* Funktion pointers for signals, needed (mostly) for gtk-- wrapper */
      /* For use of the signals, see "examples/signals.c" */
      void (*gtk_databox_motion_notify) (GtkDatabox * box, 
                                         GtkDataboxCoord * position);
      void (*gtk_databox_zoomed) (GtkDatabox * box,
				  GtkDataboxValue * top_left,
				  GtkDataboxValue * bottom_right);
      void (*gtk_databox_marked) (GtkDatabox * box, GtkDataboxCoord * marked);
      void (*gtk_databox_selection_started) (GtkDatabox * box,
					     GtkDataboxCoord * marked);
      void (*gtk_databox_selection_changed) (GtkDatabox * box,
					     GtkDataboxCoord * marked,
					     GtkDataboxCoord * select);
      void (*gtk_databox_selection_stopped) (GtkDatabox * box,
					     GtkDataboxCoord * marked,
					     GtkDataboxCoord * select);
      void (*gtk_databox_selection_cancelled) (GtkDatabox * box);

   };

   /**
    * Get type ID.
    *
    * This function returns the type ID for the GtkDatabox widget type.
    *
    * @returns Type ID
    */
   guint gtk_databox_get_type (void);

   /** 
    * Create a GtkDatabox widget.
    *
    * This function creates a GtkDatabox widget. At the beginning it will 
    * contain no data at all, so you will have to add data.  
    * 
    * It will be configured in a default way, e.g. to show rulers and 
    * scrollbars. You may want to change this configuration.
    *
    * @returns A pointer to the new GtkDatabox widget.
    *
    * @see gtk_databox_data_add
    * @see gtk_databox_set_rulers_enable
    */
   GtkWidget *gtk_databox_new (void);

   /**
    * Request a redraw of the widget.
    *
    * When you have added new data or changed "old" data, you have to tell 
    * your GtkDatabox widget to redraw the data, otherwise you will see no 
    * changes.
    *
    * @param box The GtkDatabox widget, which shall be redrawn.
    */
   void gtk_databox_redraw (GtkDatabox * box);

/* This function is used to add two arrays of float values to the 
   list of data of the GtkDatabox. One array contains the X values
   the other contains the Y values of your data points.
   GtkDatabox will not copy or backup the data. You have to make sure
   that X and Y pointers will stay valid until you remove the data
   from the GtkDatabox or until you destroy the GtkDatabox itself.
   
   You also have to define the color, size/width and type of the data
   for display. See the GtkDataboxDisplayType enumeration for the type options.
   If you want to add a grid or coordinate system, choose the appropriate
   type and use any float arrays you happen to have, because the data is 
   obsolete in these cases. 
   
   After adding data you may need to rescale the GtkDatabox to assure
   that your data will be visible on the screen. Use 
   gtk_databox_rescale and gtk_databox_rescale_with_values for this 
   purpose.
   
   The return value is the internal index of the data.
   */
   /** 
    * @brief 
    * 
    * @param box 
    * @param length 
    * @param X 
    * @param Y 
    * @param color 
    * @param type 
    * @param size 
    * 
    * @return 
    */
   gint gtk_databox_data_add (GtkDatabox * box, guint length, gfloat * X,
			      gfloat * Y, GdkColor color,
			      guint type, guint size);

   /** 
    * @brief 
    * 
    * @param box 
    * @param index 
    * 
    * @return 
    */
   GtkDataboxData *gtk_databox_data_get (GtkDatabox * box, gint index);

   /** 
    * @brief 
    * 
    * @param draw 
    * @param remove 
    * 
    * @return 
    */
   guint gtk_databox_display_type_add (GtkDataboxDrawFunc draw, GtkDataboxDrawFunc remove);

   /** 
    * @brief 
    * 
    * @param box 
    * @param cursor 
    * 
    * @return 
    */
   gint gtk_databox_set_cursor (GtkDatabox * box, GdkCursor * cursor);
   /** 
    * @brief 
    * 
    * @param box 
    * 
    * @return 
    */
   GdkCursor *gtk_databox_get_cursor (GtkDatabox * box);

/* The next two funtions are used to remove data sets from the 
   GtkDatabox. They remove the set with the given index or all sets.
   When removing only one set, X or Y data that has been used for 
   several data sets will be taken care of. */
   /** 
    * @brief 
    * 
    * @param box 
    * @param index 
    * 
    * @return 
    */
   gint gtk_databox_data_remove (GtkDatabox * box, gint index);
   /** 
    * @brief 
    * 
    * @param box 
    * 
    * @return 
    */
   gint gtk_databox_data_remove_all (GtkDatabox * box);

/* The following two functions are used to adjust the GtkDatabox to 
   your data. By default, the GtkDatabox will display data in a range 
   of (-0.5, -0.5) to (1.5, 1.5). Typically you data will not fit well into 
   this range. 
   
   By calling gtk_databox_rescale the GtkDatabox will analyse
   your data and make itself fit around the data, leaving a border of
   a few pixels for optical reasons.
   
   By calling gtk_databox_rescale_with_values you can define the 
   range of visible data yourself. This is helpful when your data
   will be undergoing changes but you know the theoretical maximum 
   minimum values. In that case you can once rescale the GtkDatabox with 
   these extrema and your data will always fit in (that is, if your
   theory is correct). Defining the extrema yourself is also useful 
   if your data contains a few "wrong" values that are far away from 
   the interesting parts. */
   /** 
    * @brief 
    * 
    * @param box 
    */
   void gtk_databox_rescale (GtkDatabox * box);
   /** 
    * @brief 
    * 
    * @param box 
    * @param min 
    * @param max 
    */
   void gtk_databox_rescale_with_values (GtkDatabox * box,
					 GtkDataboxValue min,
					 GtkDataboxValue max);

/* Automatic rescaling arranges the values so that (minX, minY) are located 
   in the left bottom of the widget and the (maxX, maxY) is found on the 
   right top. 
   By calling gtk_databox_rescale_inverted, you can opt for an inverted 
   arrangement in X and/or Y direction. */
   /** 
    * @brief 
    * 
    * @param box 
    * @param invertX 
    * @param invertY 
    */
   void gtk_databox_rescale_inverted (GtkDatabox * box, 
                                      gboolean invertX, 
                                      gboolean invertY);

/* Use the following functions to get or set the color and type 
   of your displayed data. The index is the return value of 
   gtk_databox_data_add_x_y or similar function. */
   /** 
    * @brief 
    * 
    * @param box 
    * @param index 
    * @param color 
    * 
    * @return 
    */
   gint gtk_databox_data_set_color (GtkDatabox * box, gint index,
				    GdkColor color);
   /** 
    * @brief 
    * 
    * @param box 
    * @param index 
    * @param color 
    * 
    * @return 
    */
   gint gtk_databox_data_get_color (GtkDatabox * box, gint index,
				    GdkColor * color);
   /** 
    * @brief 
    * 
    * @param box 
    * @param index 
    * @param type 
    * @param dot_size 
    * 
    * @return 
    */
   gint gtk_databox_data_set_type (GtkDatabox * box, gint index,
				   guint type, guint dot_size);
   /** 
    * @brief 
    * 
    * @param box 
    * @param index 
    * @param type 
    * @param dot_size 
    * 
    * @return 
    */
   gint gtk_databox_data_get_type (GtkDatabox * box, gint index,
				   guint * type,
				   guint * dot_size);

  
/* The following function calculates the "data coordinates" for
   a given pair of pixel coordinates. The results of this 
   calculation depend on your current zoom/scroll status, and on 
   the last rescale action, see gtk_databox_rescale and 
   gtk_databox_rescale_with_values. */
   /** 
    * @brief 
    * 
    * @param box 
    * @param coord 
    * @param value 
    */
   void gtk_databox_data_get_value (GtkDatabox * box, GtkDataboxCoord coord,
				    GtkDataboxValue * value);

/* Get the minimum X/Y and the maximum X/Y "data coordinates". These
   are the coordinates of the bottom-left and top-right corners when 
   you zoom out completely (shift + button-3). 
   The result depends on the last call of gtk_databox_rescale or
   gtk_databox_rescale_with_values */
   /** 
    * @brief 
    * 
    * @param box 
    * @param min 
    * @param max 
    */
   void gtk_databox_data_get_extrema (GtkDatabox * box, GtkDataboxValue * min,
				      GtkDataboxValue * max);
/* Get the "data coordinates" (not the pixel coordinates) of the 
   bottom-left and top-right corners of the visible data currently 
   shown in the GtkDatabox. When zoomed out completely 
   the result will be the same as for gtk_databox_data_get_extrema 
   The result depends on the last call of gtk_databox_rescale or
   gtk_databox_rescale_with_values */
   /** 
    * @brief 
    * 
    * @param box 
    * @param min 
    * @param max 
    */
   void gtk_databox_data_get_visible_extrema (GtkDatabox * box,
					      GtkDataboxValue * min,
					      GtkDataboxValue * max);


/* Even if you are happy with the default behaviour of the GtkDatabox
   you should also take a look at the following functions. 
   They might even increase you satisfaction :-) */

/* This color allows you to set the background color of the GtkDatabox. */

   /** 
    * @brief 
    * 
    * @param box 
    * @param color 
    */
   void gtk_databox_set_background_color (GtkDatabox * box, GdkColor color);

/* By default, the GtkDatabox will come with a pair of scrollbars and
   rulers. Use the following functions to get rid of them or 
   get them back again */

   /** 
    * @brief 
    * 
    * @param box 
    * @param enable 
    */
   void gtk_databox_set_rulers_enable (GtkDatabox * box, gboolean enable);
   /** 
    * @brief 
    * 
    * @param box 
    * @param enable 
    */
   void gtk_databox_set_hruler_enable (GtkDatabox * box, gboolean enable);
   /** 
    * @brief 
    * 
    * @param box 
    * @param enable 
    */
   void gtk_databox_set_vruler_enable (GtkDatabox * box, gboolean enable);
   /** 
    * @brief 
    * 
    * @param box 
    * 
    * @return 
    */
   gboolean gtk_databox_get_rulers_enable (GtkDatabox * box);
   /** 
    * @brief 
    * 
    * @param box 
    * 
    * @return 
    */
   gboolean gtk_databox_get_hruler_enable (GtkDatabox * box);
   /** 
    * @brief 
    * 
    * @param box 
    * 
    * @return 
    */
   gboolean gtk_databox_get_vruler_enable (GtkDatabox * box);
   
   /** 
    * @brief 
    * 
    * @param box 
    * @param enable 
    */
   void gtk_databox_set_scrollbars_enable (GtkDatabox * box, gboolean enable);
   /** 
    * @brief 
    * 
    * @param box 
    * @param enable 
    */
   void gtk_databox_set_hscrollbar_enable (GtkDatabox * box, gboolean enable);
   /** 
    * @brief 
    * 
    * @param box 
    * @param enable 
    */
   void gtk_databox_set_vscrollbar_enable (GtkDatabox * box, gboolean enable);
   /** 
    * @brief 
    * 
    * @param box 
    * 
    * @return 
    */
   gboolean gtk_databox_get_scrollbars_enable (GtkDatabox * box);
   /** 
    * @brief 
    * 
    * @param box 
    * 
    * @return 
    */
   gboolean gtk_databox_get_hscrollbar_enable (GtkDatabox * box);
   /** 
    * @brief 
    * 
    * @param box 
    * 
    * @return 
    */
   gboolean gtk_databox_get_vscrollbar_enable (GtkDatabox * box);

/* Selection is possible as default, but you may disable that feature */
   void gtk_databox_set_selection_enable (GtkDatabox * box, gboolean enable);
   gboolean gtk_databox_get_selection_enable (GtkDatabox * box);

/* Decide whether the selection box is filled or not, the default
   is not filled */
   void gtk_databox_set_selection_fill_enable (GtkDatabox * box,
					       gboolean enable);
   gboolean gtk_databox_get_selection_fill_enable (GtkDatabox * box);

/* Zooming into the data and back is enabled as default, but you 
   can disable zooming if it makes you nervous */
   /** 
    * @brief 
    * 
    * @param box 
    * @param enable 
    */
   void gtk_databox_set_zoom_enable (GtkDatabox * box, gboolean enable);
   /** 
    * @brief 
    * 
    * @param box 
    * 
    * @return 
    */
   gboolean gtk_databox_get_zoom_enable (GtkDatabox * box);

/* When you zoom into the GtkDatabox, you might stumble over dubious
   effects like lines or points that are not where you would expect them 
   to be. These are range overflow or limited precision problems. 
   These problems are avoided if you simply do not zoom into the data ever
   deeped, but stop at some point. This point is the so-called 
   zoom limit. It is a value between 0 and 1 (use other values at 
   your own risk), the default is 0.01. It defines the smallest fraction
   in X and Y dimension that can be zoomed to. Thus, the default allows 
   you to see everything enlarged by a maximum factor of 100. 
   
   If you think you need to play with that behaviour, use the following 
   two functions, to get and set the zoom limit. 
   
   Note: Zooming in very deep might make the GtkDatabox rather slow
   when you are displaying your data as lines. */
   /** 
    * @brief 
    * 
    * @param box 
    * @param zoom_limit 
    */
   void gtk_databox_set_zoom_limit (GtkDatabox * box, gfloat zoom_limit);
   /** 
    * @brief 
    * 
    * @param box 
    * 
    * @return 
    */
   gfloat gtk_databox_get_zoom_limit (GtkDatabox * box);

#ifdef __cplusplus
}
#endif				/* __cplusplus */

/* ------------------------------------------------------------------------- */
/* -- The drawing functions and related things are below this point -------- */
/* ------------------------------------------------------------------------- */
#include <gtkdatabox_points.h>
#include <gtkdatabox_lines.h>
#include <gtkdatabox_bars.h>
#include <gtkdatabox_grid.h>
#include <gtkdatabox_cross_simple.h>
#include <gtkdatabox_marker.h>
#include <gtkdatabox_text.h>

#endif				/* __GTK_DATABOX_H__ */
